#include "igorToyApp.h"
#include "imgui/imgui.h"
#include "imgui/imgui_impl_sdl.h"
#include "imgui/imgui_impl_opengl3.h"
#include <iostream>

// the constructor
CApp::CApp()
{
  isRunning = true;
  window = NULL;
  gl_context = NULL;
}

bool CApp::OnInit()
{
  if (SDL_Init(SDL_INIT_EVERYTHING) < 0)
    {
      return false;
    }

  // GL versions
  const char* glsl_version = "#version 150";
  SDL_GL_SetAttribute(SDL_GL_CONTEXT_FLAGS, 0);
  SDL_GL_SetAttribute(SDL_GL_CONTEXT_PROFILE_MASK, SDL_GL_CONTEXT_PROFILE_CORE);
  SDL_GL_SetAttribute(SDL_GL_CONTEXT_MAJOR_VERSION, 3);
  SDL_GL_SetAttribute(SDL_GL_CONTEXT_MINOR_VERSION, 2);

  // Creat Window
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);
  SDL_GL_SetAttribute(SDL_GL_STENCIL_SIZE, 8);


  
  SDL_WindowFlags window_flags = (SDL_WindowFlags)(SDL_WINDOW_OPENGL | SDL_WINDOW_RESIZABLE | SDL_WINDOW_ALLOW_HIGHDPI);

  window = SDL_CreateWindow("Welcome to the Terror Dome",SDL_WINDOWPOS_CENTERED,SDL_WINDOWPOS_CENTERED,1280,720,window_flags);

  gl_context = SDL_GL_CreateContext(window);
  SDL_GL_MakeCurrent(window, gl_context);

  if (window != NULL)
    {
      std::cout << "\n> Initialized Window .....\n";
    }
  else
    {
      return false;
    }

  // ImGUI stuff??
  IMGUI_CHECKVERSION();
  ImGui::CreateContext();
  ImGuiIO& io = ImGui::GetIO();
  (void) io;
  ImGui::StyleColorsDark();
  ImGui_ImplSDL2_InitForSDLRenderer(window);
  ImGui_ImplOpenGL3_Init(glsl_version);
    
  return true;
}


int CApp::OnExecute()
{
  SDL_Event event;

  if (OnInit() == false)
    {
      return -1;
    }

  while (isRunning)
    {
      while (SDL_PollEvent(&event) != 0)
	{
	  OnEvent(&event);
	}
      OnLoop();
      OnRender();
    }
  return 0;
}


void CApp::OnEvent(SDL_Event *event)
{
  if (event->type == SDL_QUIT)
    {
      isRunning = false;
    }
}

void CApp::OnLoop()
{
  
  ImGui_ImplOpenGL3_NewFrame();
  ImGui_ImplSDL2_NewFrame();
  ImGui::NewFrame();
  
  ImGui::Begin("Hello!");

  ImGui::Text("Hello, world %d", 123);

  ImGui::End();

  ImGui::Render();
    
}

void CApp::OnRender()
{

  glViewport(0, 0, 1280, 720);
  glClearColor(0.45f, 0.55f,0.60f,1.0f);
  glClear(GL_COLOR_BUFFER_BIT);
  ImGui_ImplOpenGL3_RenderDrawData(ImGui::GetDrawData());
  
}

void CApp::OnExit()
{
  std::cout << "> Exiting...";
  ImGui_ImplOpenGL3_Shutdown();
  ImGui_ImplSDL2_Shutdown();
  ImGui::DestroyContext();

  SDL_GL_DeleteContext(gl_context);
  gl_context = NULL;
  SDL_DestroyWindow(window);
  window = NULL;
  SDL_Quit();
}
